/* ***************************************************************************
 *  PD443X.ino                                                   2013-05-07  *
 * ------------------------------------------------------------------------- *
 *                                                               by kayekss  *
 *                                                                           *
 *    - Class library for OSRAM/Siemens Intelligent Display                  *
 *      PD243x/PD353x/PD443x                                                 *
 *************************************************************************** */

#include <Arduino.h>
#include "PD443X.h"

PD443X::PD443X(const uint8_t _pRST, const uint8_t _pCE0, const uint8_t _pWR,
               const uint8_t _pA2, const uint8_t _pA1, const uint8_t _pA0,
               const uint8_t _pD7, const uint8_t _pD6, const uint8_t _pD5,
               const uint8_t _pD4, const uint8_t _pD3, const uint8_t _pD2,
               const uint8_t _pD1, const uint8_t _pD0) {
  pRST = _pRST;
  pCE0 = _pCE0;
  pWR = _pWR;
  pA2 = _pA2;
  pA1 = _pA1;
  pA0 = _pA0;
  pD7 = _pD7;
  pD6 = _pD6;
  pD5 = _pD5;
  pD4 = _pD4;
  pD3 = _pD3;
  pD2 = _pD2;
  pD1 = _pD1;
  pD0 = _pD0;

  // Pin direction setup
  pinMode(pRST, OUTPUT);
  pinMode(pCE0, OUTPUT);
  pinMode(pWR, OUTPUT);
  pinMode(pA2, OUTPUT);
  pinMode(pA1, OUTPUT);
  pinMode(pA0, OUTPUT);
  pinMode(pD7, OUTPUT);
  pinMode(pD6, OUTPUT);
  pinMode(pD5, OUTPUT);
  pinMode(pD4, OUTPUT);
  pinMode(pD3, OUTPUT);
  pinMode(pD2, OUTPUT);
  pinMode(pD1, OUTPUT);
  pinMode(pD0, OUTPUT);

  // Initialize outputs
  digitalWrite(pCE0, LOW);
  digitalWrite(pWR, HIGH);

  // Reset display
  digitalWrite(pRST, LOW);
  digitalWrite(pRST, HIGH);
}

PD443X::~PD443X() {
}

void PD443X::clear() {
  for (uint8_t i = 0; i < 4; i++) {
    displayBuffer[i] = ' ';
  }

  setCommandMode();
  setAddress(0);
  setByte(0b10000000);  // Clear Entire Display
  digitalWrite(pWR, LOW);
  digitalWrite(pWR, HIGH);
  delay(1);
}

void PD443X::clear(const uint8_t br) {
  PD443X::clear();
  setBrightness(br);
}

void PD443X::setBrightness(const uint8_t br) {
  // Brightness setting value
  //   br >= 3:   100%
  //   br  = 2:    50%
  //   br  = 1:    25%
  //   br  = 0:     0%

  uint8_t br_ = br > 3 ? 3 : br;

  setCommandMode();
  setAddress(0);
  setByte(br_);
  digitalWrite(pWR, LOW);
  digitalWrite(pWR, HIGH);
}

void PD443X::writeChar(const int8_t pos, const uint8_t c) {
  if (pos < 0 || pos > 3) {  // Do nothing if out-of-range
    return;
  }

  displayBuffer[pos] = c;

  setAsciiMode();
  setAddress(pos);
  setByte(c);
  digitalWrite(pWR, LOW);
  digitalWrite(pWR, HIGH);
}

void PD443X::writeString(const char* str) {
  uint8_t i;

  if (!str) {
    return;
  }

  for (i = 0; i < 4 && str[i]; i++) {
    disp.writeChar(3 - i, (uint8_t) str[i]);
  }
  for (; i < 4; i++) {
    disp.writeChar(3 - i, ' ');
  }
}

void PD443X::scrollLeft(const uint8_t c) {
  disp.writeChar(3, displayBuffer[2]);
  disp.writeChar(2, displayBuffer[1]);
  disp.writeChar(1, displayBuffer[0]);
  disp.writeChar(0, c);
}

void PD443X::scrollRight(const uint8_t c) {
  disp.writeChar(0, displayBuffer[1]);
  disp.writeChar(1, displayBuffer[2]);
  disp.writeChar(2, displayBuffer[3]);
  disp.writeChar(3, c);
}

void PD443X::scrollString(const char* str, const uint16_t ms) {
  disp.writeString("    ");
  delay(ms);
  for (uint8_t i = 0; i < strlen(str); i++) {
    disp.scrollLeft(str[i]);
    delay(ms);
  }
  for (uint8_t i = 0; i < 4; i++) {
    disp.scrollLeft(' ');
    delay(ms);
  }
}

void PD443X::writeNumber(const int8_t pos, const uint8_t n) {
  if (n >= 0 && n <= 9) {
    writeChar(pos, n + '0');
  } else {
    writeChar(pos, '?');
  }
}

void PD443X::writeValue(const int16_t val, const int8_t offset,
                        const uint8_t digits, const bool zs) {
  const int16_t divisor[4] = { 1, 10, 100, 1000 };
  int16_t val_ = val;
  bool zs_ = zs;
  uint8_t d = 0;

  // Negate value if it is < 0
  if (val < 0) {
    val_ = -val_;
  }

  for (int8_t i = digits - 1; i >= 0; i--) {
    d = (val_ / divisor[i]) % 10;  // Slice a digit
    if (d) {
      zs_ = false;
    }

    if (!zs_ || i == 0) {
      writeNumber(offset + i, d);
    } else {
      writeChar(offset + i, ' ');
    }
  }
  // Write a minus sign
  if (val < 0) {
    disp.writeChar(offset + digits, '-');
  }
}

inline void PD443X::setCommandMode() {
  digitalWrite(pA2, LOW);
}

inline void PD443X::setAsciiMode() {
  digitalWrite(pA2, HIGH);
}

inline void PD443X::setAddress(uint8_t a) {
  uint8_t a_ = a & 0x03;

  digitalWrite(pA1, a_ & 0x02);
  digitalWrite(pA0, a_ & 0x01);
}

inline void PD443X::setByte(uint8_t b) {
  digitalWrite(pD7, b & 0x80);
  digitalWrite(pD6, b & 0x40);
  digitalWrite(pD5, b & 0x20);
  digitalWrite(pD4, b & 0x10);
  digitalWrite(pD3, b & 0x08);
  digitalWrite(pD2, b & 0x04);
  digitalWrite(pD1, b & 0x02);
  digitalWrite(pD0, b & 0x01);
}
